# SPDX-FileCopyrightText: 2024-present Leander (lpnlp202111) <leander.lacroix.dev@protonmail.com>
#
# SPDX-License-Identifier: MIT


from .dataproxy import DataProxy
from .fitparameters import FitParameters
# from . import linearmodels
